# -*- coding: utf-8 -*-
"""
Created on Thu Nov 28 09:29:45 2013

@author: bercherj
"""

from __future__ import division
from __future__ import print_function

from pylab import *
from scipy import ndimage as ndi
from scipy import signal
from numpy.fft import fft, ifft, fft2, ifft2
from scipy.ndimage import convolve as convolvend
# pour les représentations 3D
from mpl_toolkits.mplot3d.axes3d import Axes3D


## Definitions
# ============

def isodd(num):
        return num & 1 and True or False
# or num%2==1        

## 
def rect2(L,N):
    """ Rend un rectangle de taille (2L+1)x(2L+1) \n
    centré sur une dimension totale de NxN"""
    x=zeros((N,N))
    milieu=((N+1)/2,(N+1)/2) if isodd(N) else ((N)/2+1,(N)/2+1)
    x[milieu[0]-L:milieu[0]+L+1,milieu[1]-L:milieu[1]+L+1]=1    
    return x

## 
def filtre_passebande_2d(centre,largeur,dimension_totale):    
    """ Retourne la réponse en fréquence d'un filtre passe-bande
    centré sur `centre`, de demi-largeur **largeur** et symétrisé 
    en fréquence (le zéro est placé au centre de l'image en f)\n
    centre : fréquence centrale du filtre passe-bande (en points) -- les fréquences 
    normalisées correspondantes sont ainsi centre/(dimension_totale/2) \n
    largeur :  demi-bande de fréquence (en points) du filtre passe-bande\n
    dimension_totale : dimension de l'image\n 
    Auteur : jfb"""
    N=dimension_totale    
    milieu=((N+1)/2,(N+1)/2) if isodd(N) else ((N)/2+1,(N)/2+1)
    # 
    H=zeros((N,N))    
    H[milieu[0]+centre[0]-largeur:milieu[0]+centre[0]+largeur+1,milieu[1]+centre[1]-largeur:milieu[1]+centre[1]+largeur+1]=1    
    centre=-array(centre)  # Le symétrique en fréquence
    H[milieu[0]+centre[0]-largeur:milieu[0]+centre[0]+largeur+1,milieu[1]+centre[1]-largeur:milieu[1]+centre[1]+largeur+1]=1    
    return H
    
## représentation en fréquence 2D
def showfft2(X,Zero='uncentered', Freq='normalized', num=None,cmap='hot'):
    """Affiche une image, dans le domaine de Fourier 2D, \n
    la représentation étant centrée et en fréquences réduites\n
    **/!\\  la fonction *showfft2* **ne calcule pas la TF**. Les données passées sont sensées être des données fréquentielles\n
    Paramètres :
    -----------
        X : données 2D dans le domaine de Fourier \n
        Num: numéro de figure (optionnel)\n
        Zero='uncentered' par défaut, la TF passée est supposée non centrée et un fftshift est appliqué\n
        Freq='normalized' par défaut; sinon on trace en points \n
    ``Auteur: jfb``
    """
    N,M=shape(X)
    figure(num)
    if Zero!='centered':
        X=fftshift(X)
    if Freq=='normalized':
        extent=(-0.5, 0.5, -0.5, 0.5)
    else:
        extent=(-N/2, N/2, -N/2, N/2)
    im=imshow(X,aspect='auto',origin='lower', extent=extent, cmap=cmap)    
    if Freq=='normalized':   
        ticks=arange(-0.5,0.6,0.1)
        plt.xticks(ticks)
        plt.yticks(ticks)
    colorbar()
  

   
def mesh(obj3D,numfig=None,subplt=(1,1,1),cmap='hot'):
    """
    Emule le mesh de matlab : trace 
    une représentaion en 2.5D \n
    de l'objet obj3D
    Auteur: jfb
    """
    fig = plt.figure(numfig)
    ax = fig.add_subplot(subplt[0],subplt[1],subplt[2], projection='3d')
    m,n=shape(obj3D)
    mm=range(m)
    nn=range(n)
    X,Y=meshgrid(mm,nn)
    ax.plot_surface(X, Y, obj3D, rstride=max([round(m/50), 1]), cstride=max([round(n/50), 1]), cmap=cmap)
    
def saltpepper(percent=85,shape=None):
        s=numpy.random.random(size=shape)
        d=where(s>percent/100)
        out=ones(shape=shape)
        out[d]=0
        return out
        

## -----------------------------------------
# 

close('all')
#%% Lecture et affichage de l'image
B=imread('barbara.png')
figure(1)
imshow(B,cmap='gray',origin='upper')
colorbar()
